@@ -7,8 +7,8 @@ use Pugs::Runtime::Match;
 use Pugs::Grammar::StatementControl;
 use Pugs::Grammar::StatementModifier;
 use Pugs::Grammar::Expression;
-use Pugs::Grammar::Pod;
-use Pugs::Grammar::P6Rule; # our local version of Grammar::Rule.pm
+# use Pugs::Grammar::Pod;
+use Pugs::Grammar::P6Rule; 
 
 use Data::Dumper;
 
@@ -24,7 +24,7 @@ use Data::Dumper;
 *block = Pugs::Compiler::Token->compile( q(
     \{ <?ws>? <statements> <?ws>? \}
         { 
-            #print "matched block\n", Dumper( $_[0]{statements}->data ); 
+            #print "matched block", Dumper( $_[0]{statements}->data ); 
             return { 
                 bare_block => $_[0]{statements}->(),
             } 
@@ -179,22 +179,50 @@ use Data::Dumper;
     { grammar => __PACKAGE__ }
 )->code;
 
-*sub_decl_name = Pugs::Compiler::Token->compile( q(
-    ( multi | <null> ) <?ws>?
-    ( submethod | method | sub ) <?ws>? 
-    ( <?Pugs::Grammar::Term.ident>? ) 
-        { return { 
-            multi      => $_[0][0]->(),
-            statement  => $_[0][1]->(),
-            name       => $_[0][2]->(),
-        } }
+*category_name = Pugs::Compiler::Token->compile( q(
+        <Pugs::Grammar::Term.bare_ident> 
+        [   <':'> 
+            [ 
+                <before \{ > 
+                <%Pugs::Grammar::Term::hash> 
+                { return { 
+                    category   => $_[0]{'Pugs::Grammar::Term.bare_ident'}->(),
+                    name       => {
+                        'exp1' => { 
+                            'hash' => '%::_V6_GRAMMAR::' . 
+                                $_[0]{'Pugs::Grammar::Term.bare_ident'}->(), },
+                        'exp2' => $_[0]{'Pugs::Grammar::Term::hash'}
+                                ->()->{bare_block}, 
+                        'fixity' => 'postcircumfix',
+                        'op1' => { 'op' => '{' },
+                        'op2' => { 'op' => '}' },
+                    }
+                } }
+            | 
+                #<before \< > 
+                <%Pugs::Grammar::Quote::hash>
+                { return { 
+                    category   => $_[0]{'Pugs::Grammar::Term.bare_ident'}->(),
+                    name       => {
+                        'exp1' => { 
+                            'hash' => '%::_V6_GRAMMAR::' . $_[0]{'Pugs::Grammar::Term.bare_ident'}->(), },
+                        'exp2' => $_[0]{'Pugs::Grammar::Quote::hash'}->(), 
+                        'fixity' => 'postcircumfix',
+                        'op1' => { 'op' => '<' },
+                        'op2' => { 'op' => '>' },
+                    }
+                } }
+            ]
+        | 
+            { return { 
+                category   => '',
+                name       => $_[0]{'Pugs::Grammar::Term.bare_ident'}->(),
+            } }
+        ]
     |
-    ( multi ) <?ws>?
-    ( <?Pugs::Grammar::Term.ident>? ) 
         { return { 
-            multi      => $_[0][0]->(),
-            statement  => 'sub',
-            name       => $_[0][1]->(),
+            category   => '',
+            name       => '',
         } }
 ),
     { grammar => __PACKAGE__ }
@@ -213,15 +241,23 @@ use Data::Dumper;
 )->code;
 
 *sub_decl = Pugs::Compiler::Token->compile( q(
-    <sub_decl_name> <?ws>? 
+    [
+        ( multi | <null> )           <?ws>?
+        ( submethod | method | sub ) <?ws>? 
+    |
+        ( multi  ) <?ws>?
+        ( <null> )
+    ]
+    <category_name> <?ws>?
     <sub_signature> <?ws>? 
     <attribute>     <?ws>?
     <block>        
         { 
           return { 
-            multi      => $_[0]{sub_decl_name}->()->{multi},
-            term       => $_[0]{sub_decl_name}->()->{statement},
-            name       => $_[0]{sub_decl_name}->()->{name},
+            multi      => $_[0][0]->(),
+            term       => $_[0][1]->() || 'sub',
+            category   => $_[0]{category_name}->()->{category},
+            name       => $_[0]{category_name}->()->{name},
             attribute  => $_[0]{attribute}->(),
             signature  => $_[0]{sub_signature}->(),
             block      => $_[0]{block}->(),
@@ -230,33 +266,21 @@ use Data::Dumper;
     { grammar => __PACKAGE__ }
 )->code;
 
-
-*rule_decl_name = Pugs::Compiler::Token->compile( q(
+*rule_decl = Pugs::Compiler::Token->compile( q(
     ( multi | <null> )        <?ws>?
     ( rule  | regex | token ) <?ws>?
-    ( <?Pugs::Grammar::Term.ident>? ) 
-        { return { 
-            multi      => $_[0][0]->(),
-            statement  => $_[0][1]->(),
-            name       => $_[0][2]->(),
-        } }
-),
-    { grammar => __PACKAGE__ }
-)->code;
-
-
-*rule_decl = Pugs::Compiler::Token->compile( q(
-        <rule_decl_name> <?ws>?   
-        <sub_signature>  <?ws>? 
-        <attribute>      <?ws>?
-        <'{'>            <?ws>?
+    <category_name>  <?ws>?
+    <sub_signature>  <?ws>? 
+    <attribute>      <?ws>?
+    <'{'>            <?ws>?
     [
-            <Pugs::Grammar::P6Rule.rule> <?ws>?
+        <Pugs::Grammar::P6Rule.rule> <?ws>?
         <'}'>
         { return { 
-                multi      => $_[0]{rule_decl_name}->()->{multi},
-                term       => $_[0]{rule_decl_name}->()->{statement},
-                name       => $_[0]{rule_decl_name}->()->{name},
+                multi      => $_[0][0]->(),
+                term       => $_[0][1]->(),
+                category   => $_[0]{category_name}->()->{category},
+                name       => $_[0]{category_name}->()->{name},
                 attribute  => $_[0]{attribute}->(),
                 signature  => $_[0]{sub_signature}->(),
                 # pass the match tree to the emitter
@@ -320,16 +344,18 @@ use Data::Dumper;
 *statements = Pugs::Compiler::Token->compile( q(
     [ ; <?ws>? ]*
     [
-        <before <'}'> > { $::_V6_SUCCEED = 0 } 
-    |
+        <!before <'}'> >
         <statement> 
+        #{ print "02 end statement", Dumper( $_[0]{statement}->() ) } 
         <?ws>? [ ; <?ws>? ]*
         [
-            <before <'}'> > { $::_V6_SUCCEED = 0 }
-        |
+            <!before <'}'> >
+            #{ print "04 another statement\n"; }
             <statements> 
-            { return {
-                statements => [
+            { 
+                #{ print "06 return statements\n"; }
+                return {
+                    statements => [
                         $_[0]{statement}->(),
                         @{ $_[0]{statements}->()->{statements} },
                     ]
@@ -337,10 +363,17 @@ use Data::Dumper;
             }
         |
             { 
-            return {
-                statements => [ $_[0]{statement}->() ],
+                #{ print "08 return statements\n"; }
+                return {
+                    statements => [ $_[0]{statement}->() ],
             } }
         ]
+    |
+            { 
+                #{ print "10 return statements\n"; }
+                return {
+                    statements => [],
+            } }
     ]
 ),
     { grammar => __PACKAGE__ }
