@@ -4,6 +4,7 @@ use warnings;
 use base qw(Pugs::Grammar::BaseCategory);
 use Pugs::Runtime::Match;
 use Pugs::Compiler::Token;
+use Pugs::Grammar::Term;
 use Text::Balanced;
 
 sub single_quoted {
@@ -25,25 +26,6 @@ sub single_quoted {
     } );
 }
 
-sub double_quoted {
-    # TODO - parse and interpolate variables
-    my $grammar = shift;
-    return $grammar->no_match(@_) unless $_[0];
-    my $pos = $_[1]{p} || 0;
-    my $s = substr( $_[0], $pos );
-    my ($extracted,$remainder) = Text::Balanced::extract_delimited( '"' . $s, '"' );
-    return $grammar->no_match(@_) unless length($extracted) > 0;
-    $extracted = substr( $extracted, 1, -1 );
-    return Pugs::Runtime::Match->new( { 
-        bool    => \1,
-        str     => \$_[0],
-        match   => [],
-        from    => \$pos,
-        to      => \( length($_[0]) - length($remainder) ),
-        capture => \$extracted,
-    } );
-}
-
 sub angle_quoted {
     my $grammar = shift;
     return $grammar->no_match(@_) unless $_[0];
@@ -62,24 +44,65 @@ sub angle_quoted {
     } );
 }
 
-sub double_angle_quoted {
-    my $grammar = shift;
-    return $grammar->no_match(@_) unless $_[0];
-    my $pos = $_[1]{p} || 0;
-    my $s = substr( $_[0], $pos );
-    my ($extracted,$remainder) = Text::Balanced::extract_bracketed( '«' . $s, '«..»' );
-    # print "double_angle_quoted at $pos: «$extracted»\n";
-    return $grammar->no_match(@_) unless length($extracted) > 0;
-    $extracted = substr( $extracted, 1, -1 );
-    return Pugs::Runtime::Match->new( { 
-        bool    => \1,
-        str     => \$_[0],
-        match   => [],
-        from    => \$pos,
-        to      => \( length($_[0]) - length($remainder) ),
-        capture => \$extracted,
-    } );
-}
+*double_quoted_expression = Pugs::Compiler::Token->compile(q(
+        <before [ <'$'> | <'@'> | <'%'> | <'&'> ] >
+        <Pugs::Grammar::Term.parse>
+        { return $/{'Pugs::Grammar::Term.parse'}() }
+))->code;
+
+*double_quoted_text = Pugs::Compiler::Token->compile(q(
+    (
+        <!before [ <'$'> | <'@'> | <'%'> | <'&'> | <'"'> ] >
+        [ <'\"'> | . ]
+    )+
+    { return { double_quoted => $/() ,} }
+))->code;
+
+*double_quoted = Pugs::Compiler::Token->compile(q(
+     
+    [  $<q1> := <double_quoted_expression>
+    |  $<q1> := <double_quoted_text>
+    ]
+    
+    [   $<q2> := <double_quoted> 
+        { return { 
+            exp1 => $/{q1}(), 
+            exp2 => $/{q2}(),
+            'fixity' => 'infix',
+            'op1' => {
+                'op' => '~',
+            }
+        } } 
+    |   { return $/{q1}() } 
+    ]
+))->code;
+
+*double_angle_quoted_text = Pugs::Compiler::Token->compile(q(
+    (
+        <!before [ <'$'> | <'@'> | <'%'> | <'&'> | <'»'> | \>\> ] >
+        [ <'\»'> | . ]
+    )+
+    { return { double_quoted => $/() ,} }
+))->code;
+
+*double_angle_quoted = Pugs::Compiler::Token->compile(q(
+     
+    [  $<q1> := <double_quoted_expression>
+    |  $<q1> := <double_angle_quoted_text>
+    ]
+    
+    [   $<q2> := <double_angle_quoted> 
+        { return { 
+            exp1 => $/{q1}(), 
+            exp2 => $/{q2}(),
+            'fixity' => 'infix',
+            'op1' => {
+                'op' => '~',
+            }
+        } } 
+    |   { return $/{q1}() } 
+    ]
+))->code;
 
 BEGIN {
 
@@ -90,8 +113,10 @@ BEGIN {
         ) );
     __PACKAGE__->add_rule(
         q(") =>  q( 
-            <Pugs::Grammar::Quote.double_quoted>
-            { return { double_quoted => $/{'Pugs::Grammar::Quote.double_quoted'}->() ,} }
+            <Pugs::Grammar::Quote.double_quoted> <'"'>
+            { return $/{'Pugs::Grammar::Quote.double_quoted'}->() }
+          | <'"'>
+            { return { double_quoted => '' } }
         ) );
     __PACKAGE__->add_rule(
         q(<) => q(
@@ -103,7 +128,15 @@ BEGIN {
         ) );
     __PACKAGE__->add_rule(
         q(«) => q(
-            <Pugs::Grammar::Quote.double_angle_quoted>
+            <Pugs::Grammar::Quote.double_angle_quoted> <'»'>
+            { return { 
+                    double_angle_quoted => $/{'Pugs::Grammar::Quote.double_angle_quoted'}->(),
+                } 
+            }
+        ) );
+    __PACKAGE__->add_rule(
+        q(<<) => q(
+            <Pugs::Grammar::Quote.double_angle_quoted> \>\>
             { return { 
                     double_angle_quoted => $/{'Pugs::Grammar::Quote.double_angle_quoted'}->(),
                 } 
